use std::path::{Path, PathBuf};

use actix_web::{HttpRequest, HttpResponse};

pub struct ProxyMirror {
    mirrors: Vec<String>,
    data_dir: String,
    no_cache: regex::Regex,
}

impl ProxyMirror {
    pub fn new(mirrors: Vec<String>, data_dir: &str, no_cache: &str) -> Self {
        Self {
            mirrors,
            data_dir: data_dir.to_string(),
            no_cache: regex::Regex::new(no_cache).unwrap(),
        }
    }

    pub fn create_cache_dir(dir: &Path) {
        if dir.is_file() {
            let tmp_file_path = dir.with_extension("tmp");
            std::fs::rename(dir, &tmp_file_path).expect("Failed to rename file");

            std::fs::create_dir_all(dir).expect("Failed to create directory");

            let index_file_path = dir.join("index");
            std::fs::rename(&tmp_file_path, index_file_path)
                .expect("Failed to move file into directory");
        } else {
            std::fs::create_dir_all(dir).unwrap();
        }
    }

    pub async fn get(&self, path: &str, req: &HttpRequest) -> Option<HttpResponse> {
        let p = std::path::Path::new(&path[1..]);
        let p = std::path::Path::new(&self.data_dir).join(p);

        if !self.no_cache.is_match(path) {
            Self::create_cache_dir(p.parent().unwrap());

            if p.exists() {
                // todo : refresh caches
                log::info!("Returning {path} from cache");
                if p.is_dir() {
                    return Some(
                        actix_files::NamedFile::open_async(p.join("index"))
                            .await
                            .ok()?
                            .into_response(req),
                    );
                }
                return Some(
                    actix_files::NamedFile::open_async(&p)
                        .await
                        .ok()?
                        .into_response(req),
                );
            }
        }

        for mirror in &self.mirrors {
            log::info!("Fetching {path} from mirrors");
            let url = format!("{mirror}{path}");
            let res = self.get_url(&url, &p).await;
            if let Some(res) = res {
                if res.status().is_success() {
                    return Some(res);
                }
            }
        }
        None
    }

    pub async fn get_url(&self, path: &str, save: &PathBuf) -> Option<HttpResponse> {
        log::info!("Fetching {path}");
        let response = reqwest::get(path).await.unwrap();
        let status_code = response.status();
        let body_bytes = response.bytes().await.ok()?;
        if status_code.is_success() {
            std::fs::write(save, &body_bytes).unwrap();
        }
        let mut http_response = HttpResponse::build(
            actix_web::http::StatusCode::from_u16(status_code.as_u16()).unwrap(),
        );
        let http_response = http_response.body(body_bytes);
        Some(http_response)
    }
}
